import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  select,
  axisLeft,
  axisBottom,
  csv,
  arc,
  pie,
  scaleBand,
  scaleLinear,
  max,
  format,
} from 'd3';

// URL to the CSV data file
const csvUrl =
  'https://gist.githubusercontent.com/anvesh2129/fd41c8b9e4ac2be354714cbdef9317f0/raw/8f901bab0a5ab008c71e1885933e50f3aee55e30/bikes_ind.csv';

// Width and height of the SVG container
const width = 1200;
const height = 600;

// Margins for the SVG container
const margin = {
  top: 10,
  right: 225,
  bottom: 200,
  left: 175,
};

// Define the main React component
const App = () => {
  // Declare a state variable for the data and initialize it to null
  const [data, setData] = useState(null);

  // Use the useEffect hook to fetch the data and parse it into the desired format
  useEffect(() => {
    // Define a function to transform each row of the CSV data
    const row = (d) => {
      // Add a count field to each row to represent the number of instances of that brand
      d.count = 1;
      return d;
    };

    // Use the d3.csv function to fetch the CSV data and parse it using the row function
    csv(csvUrl, row).then((data) => {
      // Count the number of instances of each brand using an object
      const brandCounts = {};
      data.forEach((d) => {
        const brand = d.brand;
        brandCounts[brand] =
          (brandCounts[brand] || 0) + 1;
      });
      // Transform the brandCounts object into an array of objects with brand and count fields
      setData(
        Object.entries(brandCounts).map(
          ([brand, count]) => ({
            brand,
            count,
          })
        )
      );
    });
  }, []);

  // If the data hasn't loaded yet, show a "Loading..." message
  if (!data) {
    return <pre>Loading...</pre>;
  }

  // Calculate the inner dimensions of the SVG container (i.e. the dimensions of the chart area)
  const innerHeight =
    height - margin.top - margin.bottom - 100;
  const innerWidth =
    width - margin.left - margin.right;

  // Create a scale for the y-axis using scaleBand
  const yScale = scaleBand()
    .domain(data.map((d) => d.brand)) // Set the domain to the list of brands
    .range([0, innerHeight]) // Set the range to the height of the chart area
    .padding(0.3); // Set the padding between bars to 0.3 times the width of each bar

  // Create a scale for the x-axis using scaleLinear
  const xScale = scaleLinear()
    .domain([0, max(data, (d) => d.count) * 1.09]) // Set the domain to [0, max count * 1.09] to leave room for labels
    .range([0, innerWidth]); // Set the range to the width of the chart area

  return (
    // create a SVG with defined width and height
    <svg width={width} height={height}>
      <g
        // translate the group element to the specified margin
        transform={`translate(${margin.left},${margin.top})`}
      >
        {/* create vertical gridlines */}
        {xScale.ticks().map((tickValue) => (
          <g
            key={tickValue}
            // translate the line to the x position of the tick value
            transform={`translate(${xScale(
              tickValue
            )},0)`}
          >
            {/* draw a vertical line */}
            <line
              y2={innerHeight}
              stroke="#ccc"
            />
            {/* add a label for the tick value */}
            <text
              style={{ textAnchor: 'middle' }}
              dy=".70em"
              y={innerHeight + 3}
              fill="axisColor"
            >
              {tickValue}
            </text>
          </g>
        ))}
        {/* create horizontal gridlines */}
        {yScale.domain().map((tickValue) => (
          <g
            key={tickValue}
            className="tick"
            // translate the line to the y position of the tick value
            transform={`translate(0, ${yScale(
              tickValue
            )})`}
          >
            {/* draw a horizontal line */}
            <line x2={innerWidth} stroke="#ccc" />
            {/* add a label for the tick value */}
            <text
              x={-10}
              dy=".65em"
              style={{ textAnchor: 'end' }}
            >
              {tickValue}
            </text>
          </g>
        ))}
        {/* add a label for the y-axis */}
        <text
          x={-120}
          y={innerHeight / 2}
          textAnchor="middle"
          fontWeight="bold"
          fill="gray"
          // rotate the label by 90 degrees
          transform={`rotate(-90, ${-120}, ${
            innerHeight / 2
          })`}
        >
          Bike Companies
        </text>
        {/* add a label for the x-axis */}
        <text
          x={innerWidth / 2}
          y={innerHeight + 40}
          textAnchor="middle"
          fontWeight="bold"
          fill="gray"
        >
          No. of Models
        </text>
        {/* draw the bars for each data point */}
        {data.map((d) => (
          <rect
            key={d.brand}
            x={0}
            y={yScale(d.brand)}
            // set the width of the bar based on the count value
            width={xScale(d.count)}
            // set the height of the bar based on the yScale bandwidth
            height={yScale.bandwidth()}
            fill="#093b23e8"
          />
        ))}
      </g>
    </svg>
  );
};
// render the app to the root element
const rootElement = document.getElementById(
  'root'
);
ReactDOM.render(<App />, rootElement);
